@@ -66,10 +66,10 @@ __all__       = ('Binding', 'WeakBinding', 'RaisingWeakBinding',
                  'CannotWeakReferenceError', 'GarbageCollectedError')
 
 
-import types
+from types        import FunctionType, MethodType
 import weakref
 
-from notify.utils import *
+from notify.utils import DummyReference
 
 
 
@@ -96,8 +96,16 @@ class Binding (object):
 
         See also C{L{wrap}} class method for a different way of creating bindings.
 
-        @raises TypeError: if C{callable_object} is not callable or C{arguments} is not
-                           iterable.
+        @param  callable_object: the callable object that will be invoked by this binding
+                                 from C{L{__call__}} method.
+        @type   callable_object: callable
+
+        @param  arguments:       optional list of argument for C{callable_object} that
+                                 will be prepended to call arguments.
+        @type   arguments:       iterable
+
+        @raises TypeError:       if C{callable_object} is not callable or C{arguments} is
+                                 not iterable.
         """
 
         if not callable (callable_object):
@@ -120,7 +128,7 @@ class Binding (object):
         self._arguments = arguments
 
 
-    def wrap (_class, callable_object, arguments = ()):
+    def wrap (cls, callable_object, arguments = ()):
         """
         Return a callable with semantics of the binding class this method is called for.
         I{If necessary} (e.g. if C{arguments} tuple is not empty), this method creates a
@@ -132,6 +140,14 @@ class Binding (object):
         This is the preferred method of creating bindings.  It is generally more memory-
         and call-time-efficient since in some cases no new objects are created at all.
 
+        @param  callable_object: the callable object that will be invoked by this binding
+                                 from C{L{__call__}} method.
+        @type   callable_object: callable
+
+        @param  arguments:       optional list of argument for C{callable_object} that
+                                 will be prepended to call arguments.
+        @type   arguments:       iterable
+
         @rtype:            callable
 
         @raises TypeError: if C{callable_object} is not callable or C{arguments} is not
@@ -139,7 +155,7 @@ class Binding (object):
         """
 
         if arguments is not ():
-            return _class (callable_object, arguments)
+            return cls (callable_object, arguments)
         else:
             if not callable (callable_object):
                 raise TypeError ("`callable_object' must be callable")
@@ -158,7 +174,7 @@ class Binding (object):
 
         @note:  Never override C{im_self} property, override this method instead.
 
-        @rtype: object
+        @rtype: C{object}
         """
 
         return self._object
@@ -184,7 +200,7 @@ class Binding (object):
 
         @note:  Never override C{im_class} property, override this method instead.
 
-        @rtype: class or type
+        @rtype: C{types.ClassType} or C{type}
         """
 
         return self._class
@@ -197,7 +213,7 @@ class Binding (object):
 
         @note:  Never override C{im_args} property, override this method instead.
 
-        @rtype: tuple
+        @rtype: C{tuple}
         """
 
         return self._arguments
@@ -210,7 +226,10 @@ class Binding (object):
         I{prepended} to arguments of this function before being passed to the wrapped
         callable.
 
-        @rtype:            object
+        @param  arguments: optional call arguments.
+
+        @rtype:            C{object}
+
         @raises exception: whatever wrapped method raises, if anything.
         """
 
@@ -229,7 +248,7 @@ class Binding (object):
         wrap equal methods and have equal L{argument lists <im_args>}.  A binding with an
         empty argument list is also equal to its wrapped method or function.
 
-        @rtype: bool
+        @rtype: C{bool}
         """
 
         if self is other:
@@ -247,7 +266,7 @@ class Binding (object):
                 return self._get_arguments () is ()
 
         else:
-            if isinstance (other, types.FunctionType):
+            if isinstance (other, FunctionType):
                 return (self.im_func is other
                         and self.im_self is None
                         and self.im_class is None
@@ -261,7 +280,7 @@ class Binding (object):
         """
         Determine if C{self} is not equal to C{other}.  See L{__eq__} for details.
 
-        @rtype: bool
+        @rtype: C{bool}
         """
 
         equal = self.__eq__(other)
@@ -278,7 +297,7 @@ class Binding (object):
         mainly exists for L{weak bindings <WeakBinding>}, for which it returns C{False}
         if binding’s object has been garbage-collected.
 
-        @rtype:   bool
+        @rtype:   C{bool}
         @returns: Always C{True} for this class.
         """
 
@@ -335,7 +354,14 @@ class Binding (object):
 
 
 
-BindingCompatibleTypes = (types.MethodType, Binding)
+BindingCompatibleTypes = (MethodType, Binding)
+"""
+Types ‘compatible’ with C{L{Binding}} to certain extent.  These include
+C{types.MethodType} and C{Binding} itself.  Both have C{im_self}, C{im_class} and
+C{im_func} properties (though C{im_args} is unique to C{Binding}.)  Rationale to have this
+variable is similar to that of C{weakref.ProxyTypes}.  In particular, implementation
+itself uses the variable several times in calls to C{isinstance} function.
+"""
 
 
 
@@ -382,6 +408,18 @@ class WeakBinding (Binding):
 
         See also C{L{wrap}} class method for a different way of creating weak bindings.
 
+        @param  callable_object: the callable object that will be invoked by this binding
+                                 from C{L{__call__}} method.
+        @type   callable_object: callable
+
+        @param  arguments:       optional list of argument for C{callable_object} that
+                                 will be prepended to call arguments.
+        @type   arguments:       iterable
+
+        @param  callback:        optional callable that will be called if binding’s object
+                                 is garbage-collected.
+        @type   callback:        callable or C{None}
+
         @raises TypeError:                if C{callable_object} is not callable or
                                           C{arguments} is not iterable.
         @raises CannotWeakReferenceError: if C{callable_object} is a bound method, but
@@ -403,13 +441,13 @@ class WeakBinding (Binding):
             self._object = _NONE_REFERENCE
 
 
-    def wrap (_class, callable_object, arguments = (), callback = None):
+    def wrap (cls, callable_object, arguments = (), callback = None):
         # Inherit documentation somehow?
         if (arguments is not ()
             or (    isinstance (callable_object, BindingCompatibleTypes)
                 and not isinstance (callable_object, WeakBinding)
                 and callable_object.im_self is not None)):
-            return _class (callable_object, arguments, callback)
+            return cls (callable_object, arguments, callback)
         else:
             return callable_object
 
@@ -431,7 +469,10 @@ class WeakBinding (Binding):
         Like L{Binding.__call__}, but account for garbage-collected objects.  If object
         has been garbage-collected, then do nothing and return C{None}.
 
-        @rtype:            object
+        @param  arguments: optional call arguments.
+
+        @rtype:            C{object}
+
         @raises exception: whatever wrapped method raises, if anything.
         """
 
@@ -461,7 +502,7 @@ class WeakBinding (Binding):
         the binding was created without an object (i.e. with C{None}) to begin with, this
         method will never be called at all.
 
-        @rtype: object
+        @rtype: C{object}
         """
 
         return None
@@ -482,7 +523,7 @@ class WeakBinding (Binding):
         if binding is in its initial and fully functional state, but for weak bindings it
         means exactly what is stated in the previous statement.
 
-        @rtype: bool
+        @rtype: C{bool}
         """
 
         return self._object is not None
@@ -525,7 +566,7 @@ class CannotWeakReferenceError (TypeError):
 class GarbageCollectedError (RuntimeError):
 
     """
-    Exception thrown when calling an instance of L{RaisingWeakBinding} with a
+    Exception thrown when calling an instance of C{L{RaisingWeakBinding}} with a
     garbage-collected object.
     """
 
