@@ -36,10 +36,10 @@ __all__       = ('AbstractCondition', 'AbstractStateTrackingCondition',
 
 import weakref
 
-from notify.base   import *
-from notify.gc     import *
-from notify.signal import *
-from notify.utils  import *
+from notify.base   import AbstractValueObject
+from notify.gc     import AbstractGCProtector
+from notify.signal import CleanSignal
+from notify.utils  import raise_not_implemented_exception, DummyReference
 
 
 
@@ -51,6 +51,12 @@ class AbstractCondition (AbstractValueObject):
     Abstract base class of condition hierarchy tree.  All conditions derive from this
     class, so you should use C{isinstance (..., AbstractCondition)}, not
     C{isinstance (..., Condition)}.
+
+    @cvar TRUE:
+    A condition that is always true and never changes its state.
+
+    @cvar FALSE:
+    A condition that is always false and never changes its state.
     """
 
     __slots__ = ()
@@ -73,7 +79,10 @@ class AbstractCondition (AbstractValueObject):
         words, this static method returns a condition which is either always true or
         always false.
 
-        @rtype: AbstractCondition
+        @param state: desired state of the returned condition (coerced to C{bool} first.)
+        @type  state: C{object}
+
+        @rtype:       C{AbstractCondition}
         """
 
         if state:
@@ -86,11 +95,11 @@ class AbstractCondition (AbstractValueObject):
 
     def __nonzero__(self):
         """
-        Return the state of the condition.  Return value is the same as that of C{L{get}}
-        method.  Existance of C{__nonzero__} method simplifies condition usage in
-        C{if}-like statements.
+        Return the state of the condition.  Return value is the same as that of
+        C{L{state}} property or C{L{get}} method.  Existance of C{__nonzero__} method
+        simplifies condition usage in C{if}-like statements.
 
-        @rtype:   bool
+        @rtype:   C{bool}
         @returns: State of the condition.
         """
 
@@ -112,13 +121,31 @@ class AbstractCondition (AbstractValueObject):
         and that it is an instance of C{AbstractCondition} or a subclass.  In particular,
         the returned object may or may not be identical to an existing one.
 
-        @rtype: AbstractCondition
+        @rtype: C{AbstractCondition}
         """
 
         return _Not (self)
 
 
     def __and__(self, other):
+        """
+        Return a condition, whose state is always logical ‘and’ function of this condition
+        state and C{other} state.
+
+        You don’t need this if you just want to test condition state: both
+        C{condition1 and condition2} and C{condition1 & condition2} have the same logical
+        value, but former doesn’t involve object creation.  However, the returned object
+        has its own ‘changed’ signal.  So, you should use this method if you need a
+        I{trackable} condition, not one-time value.
+
+        @note:
+        There is no guarantee on the returned object except as noted above about its state
+        and that it is an instance of C{AbstractCondition} or a subclass.  In particular,
+        the returned object may or may not be identical to an existing one.
+
+        @rtype: C{AbstractCondition}
+        """
+
         if isinstance (other, AbstractCondition):
             # Note: similar checks for `self' are performed in appropriate classes.
 
@@ -134,6 +161,24 @@ class AbstractCondition (AbstractValueObject):
 
 
     def __or__(self, other):
+        """
+        Return a condition, whose state is always logical ‘or’ function of this condition
+        state and C{other} state.
+
+        You don’t need this if you just want to test condition state: both
+        C{condition1 or condition2} and C{condition1 | condition2} have the same logical
+        value, but former doesn’t involve object creation.  However, the returned object
+        has its own ‘changed’ signal.  So, you should use this method if you need a
+        I{trackable} condition, not one-time value.
+
+        @note:
+        There is no guarantee on the returned object except as noted above about its state
+        and that it is an instance of C{AbstractCondition} or a subclass.  In particular,
+        the returned object may or may not be identical to an existing one.
+
+        @rtype: C{AbstractCondition}
+        """
+
         if isinstance (other, AbstractCondition):
             # Note: similar checks for `self' are performed in appropriate classes.
 
@@ -149,6 +194,23 @@ class AbstractCondition (AbstractValueObject):
 
 
     def __xor__(self, other):
+        """
+        Return a condition, whose state is always logical ‘xor’ function of this condition
+        state and C{other} state.
+
+        The returned object has its own ‘changed’ signal, so primary use of this method is
+        to receive a I{trackable} condition, not one-time value.  However, since there is
+        no logical ‘xor’ function in Python, you may use it to compute a one-time value,
+        though it is not efficient.
+
+        @note:
+        There is no guarantee on the returned object except as noted above about its state
+        and that it is an instance of C{AbstractCondition} or a subclass.  In particular,
+        the returned object may or may not be identical to an existing one.
+
+        @rtype: C{AbstractCondition}
+        """
+
         if isinstance (other, AbstractCondition):
             # Note: similar checks for `self' are performed in appropriate classes.
 
@@ -184,7 +246,7 @@ class AbstractCondition (AbstractValueObject):
         and that it is an instance of C{AbstractCondition} or a subclass.  In particular,
         the returned object may or may not be identical to an existing one.
 
-        @rtype: AbstractCondition
+        @rtype: C{AbstractCondition}
         """
 
         if  (   isinstance (true_condition,  AbstractCondition)
@@ -217,6 +279,9 @@ class AbstractStateTrackingCondition (AbstractCondition):
         Initialize a new condition with specified C{initial_state}.  The state may be not
         C{True} or C{False}, but it will be converted to either of these two values with
         C{bool} standard function.
+
+        @param initial_state: initial state for the new condition.
+        @type  initial_state: C{object}
         """
 
         super (AbstractStateTrackingCondition, self).__init__()
@@ -236,8 +301,8 @@ class AbstractStateTrackingCondition (AbstractCondition):
             return False
 
 
-    def _generate_derived_type_dictionary (self_class, options):
-        for attribute in (super (AbstractStateTrackingCondition, self_class)
+    def _generate_derived_type_dictionary (cls, options):
+        for attribute in (super (AbstractStateTrackingCondition, cls)
                           ._generate_derived_type_dictionary (options)):
             if attribute[0] != 'get':
                 yield attribute
@@ -248,13 +313,13 @@ class AbstractStateTrackingCondition (AbstractCondition):
         if 'getter' in options:
             if object is not None:
                 exec (('def __init__(self, %s):\n'
-                       '    self_class.__init__(self, getter (%s))\n'
+                       '    cls.__init__(self, getter (%s))\n'
                        '    %s = %s')
                       % (object, object, AbstractValueObject._get_object (options), object)) \
                       in options, functions
             else:
                 exec ('def __init__(self):\n'
-                      '    self_class.__init__(self, getter (self))\n') in options, functions
+                      '    cls.__init__(self, getter (self))\n') in options, functions
 
             exec (('def resynchronize_with_backend (self):\n'
                    '    self._set (getter (%s))')
@@ -263,13 +328,13 @@ class AbstractStateTrackingCondition (AbstractCondition):
         else:
             if object is not None:
                 exec (('def __init__(self, %s, initial_state):\n'
-                       '    self_class.__init__(self, initial_state)\n'
+                       '    cls.__init__(self, initial_state)\n'
                        '    %s = %s')
                       % (object, AbstractValueObject._get_object (options), object)) \
                       in options, functions
             else:
                 exec ('def __init__(self, initial_state):\n'
-                      '    self_class.__init__(self, initial_state)\n') in options, functions
+                      '    cls.__init__(self, initial_state)\n') in options, functions
 
         for function in functions.iteritems ():
             yield function
@@ -298,7 +363,16 @@ class Condition (AbstractStateTrackingCondition):
 
 
     def set (self, value):
-        self._set (value)
+        """
+        Set the state of the condition to C{value}.  Value is first converted using
+        C{bool} function, so it doesn’t need to be either C{True} or C{False}.
+
+        @param value: new I{state} of the condition (parameter name is just kept the same
+                      as in the base class.)
+        @type  value: C{object}
+        """
+
+        return self._set (value)
 
 
 
@@ -310,6 +384,10 @@ class PredicateCondition (AbstractStateTrackingCondition):
     recomputed, so you need to actively C{L{update}} it.  For an automated way, see
     C{L{AbstractVariable.predicate <variable.AbstractVariable.predicate>}} method.
 
+    Predicate conditions remember their state, so C{predicate} is called only from
+    C{L{__init__}} and C{L{update}} methods.  They also don’t reference last value they
+    were updated for.
+
     Advantage of using predicate conditions over all-purpose mutable ones and setting its
     state to the same predicate over different objects is encapsulation.  Predicate
     conditions remember the way to evaluate their state.
@@ -319,6 +397,22 @@ class PredicateCondition (AbstractStateTrackingCondition):
 
 
     def __init__(self, predicate, initial_object):
+        """
+        Initialize a new predicate conditon, setting its initial state to
+        C{predicate (initial_object)}.  C{predicate} is stored to be invoked also on any
+        subsequent calls to C{L{update}} method.  Return value of C{predicate} is always
+        coerced using C{bool} function, so it needn’t return a boolean value.
+
+        @param predicate:      a callable accepting one argument used to compute
+                               condition’s state.
+
+        @param initial_object: the value to be passed to C{predicate} to find initial
+                               state.
+        @type  initial_object: C{object}
+
+        @raises exception:     whatever C{predicate} raises, if anything.
+        """
+
         if not callable (predicate):
             raise TypeError ('predicate must be callable')
 
@@ -327,7 +421,20 @@ class PredicateCondition (AbstractStateTrackingCondition):
 
 
     def update (self, object):
-        self._set (self.__predicate (object))
+        """
+        Recompute condition state for new C{object}.  This method invokes stored
+        C{predicate} (as passed to C{L{__init__}}) to calculate new state.
+
+        @param object:     the value to be passed to the condition’s predicate.
+        @type  object:     C{object}
+
+        @rtype:            C{bool}
+        @returns:          Whether the state has changed as a result of recomputing.
+
+        @raises exception: whatever C{predicate} raises, if anything.
+        """
+
+        return self._set (self.__predicate (object))
 
 
     def _additional_description (self, formatter):
@@ -351,7 +458,7 @@ class WatcherCondition (AbstractStateTrackingCondition):
     condition from A to B and those have different states, watcher’s ‘changed’ signal will
     get emitted.  With manual reconnecting you’d need to track that case specially.
 
-    Watcher condition that doesn’t watch anything at the moment always has state C{False}.
+    Watcher condition that doesn’t watch anything at the moment always has C{False} state.
 
     @see:  variable.WatcherVariable
     """
@@ -362,8 +469,17 @@ class WatcherCondition (AbstractStateTrackingCondition):
 
     def __init__(self, condition_to_watch = None):
         """
-        Create a new wather condition, watching C{condition_to_watch} initially.  The only
-        argument is optional and can be omitted or set to C{None}.
+        Create a new watcher condition, watching C{condition_to_watch} initially.  The
+        only argument is optional and can be omitted or set to C{None}.
+
+        @param  condition_to_watch: condition to watch (copy state) initially.
+        @type   condition_to_watch: C{L{AbstractCondition}} or C{None}
+
+        @raises TypeError:          if C{condition_to_watch} is not an instance of
+                                    C{L{AbstractCondition}} and not C{None}.
+        @raises ValueError:         if C{condition_to_watch} is this condition.
+
+        @see:                       C{L{watch}}
         """
 
         super (WatcherCondition, self).__init__(False)
@@ -373,10 +489,26 @@ class WatcherCondition (AbstractStateTrackingCondition):
 
 
     def watch (self, condition_to_watch):
-        if (condition_to_watch is not None
-            and (   not isinstance (condition_to_watch, AbstractCondition)
-                 or condition_to_watch is self)):
-            raise TypeError ('can only watch other conditions')
+        """
+        Watch C{condition_to_watch} instead of whatever is watched now.  This method
+        disconnects internal handler from the old condition and connects it to the new
+        one, if new is not C{None}.  Watching a different condition might change own
+        state, in which case ‘changed’ signal will get emitted.
+
+        @param  condition_to_watch: new condition to watch (copy state.)
+        @type   condition_to_watch: C{L{AbstractCondition}} or C{None}
+
+        @raises TypeError:          if C{condition_to_watch} is not an instance of
+                                    C{L{AbstractCondition}} and not C{None}.
+        @raises ValueError:         if C{condition_to_watch} is this condition.
+        """
+
+        if condition_to_watch is not None:
+            if isinstance (condition_to_watch, AbstractCondition):
+                if condition_to_watch is self:
+                    raise ValueError ('cannot watch self')
+            else:
+                raise TypeError ('can only watch other conditions')
 
         watched_condition = self.__get_watched_condition ()
         if watched_condition is not None:
@@ -403,6 +535,20 @@ class WatcherCondition (AbstractStateTrackingCondition):
             return None
 
 
+    watched_condition = property (__get_watched_condition,
+                                  lambda self, condition: self.watch (condition),
+                                  doc = ("""
+                                         The currently watched condition or C{None} if
+                                         nothing is being watched.  Setting this property
+                                         is identical to calling C{L{watch}}; this
+                                         duplication of functionality is intentional,
+                                         since you cannot perform assignments in lambda
+                                         functions.
+
+                                         @type:  C{L{AbstractCondition}} or C{None}
+                                         """))
+
+
     def _create_signal (self):
         if self.__get_watched_condition () is not None:
             AbstractGCProtector.default.protect (self)
@@ -605,7 +751,7 @@ class _Not (AbstractStateTrackingCondition):
 
 class _Binary (AbstractCondition):
 
-    __slots__ = ('_Binary__condition1', '_Binary__condition2', '_num_true_conditions')
+    __slots__ = ('_Binary__condition1', '_Binary__condition2', '_term_state')
 
 
     def __init__(self, condition1, condition2):
@@ -614,16 +760,17 @@ class _Binary (AbstractCondition):
         self.__condition1 = weakref.ref (condition1, self.__on_usage_change)
         self.__condition2 = weakref.ref (condition2, self.__on_usage_change)
 
-        condition1.changed.connect (self._on_term_change)
-        condition2.changed.connect (self._on_term_change)
+        condition1.changed.connect (self._on_term1_change)
+        condition2.changed.connect (self._on_term2_change)
 
-        # Note: this doesn't allow for non-symmetric condition implementation, but we
-        # don't need those anyway and this class is private.
-        self._num_true_conditions = condition1.get () + condition2.get ()
+        self._term_state = condition1.get () + 2 * condition2.get ()
 
 
     # For efficiency reasons, descendants must override fully.
-    def _on_term_change (self, new_state):
+    def _on_term1_change (self, new_state):
+        raise_not_implemented_exception (self)
+
+    def _on_term2_change (self, new_state):
         raise_not_implemented_exception (self)
 
 
@@ -643,13 +790,11 @@ class _Binary (AbstractCondition):
                 AbstractGCProtector.default.unprotect (self)
         else:
             if object is self.__condition1:
-                self.__condition1 = _get_dummy_reference (self._num_true_conditions
-                                                          - self.__condition2 ().get ())
+                self.__condition1 = _get_dummy_reference (self._term_state & 1)
                 if self._has_signal () and isinstance (self.__condition2, DummyReference):
                     AbstractGCProtector.default.unprotect (self)
             else:
-                self.__condition2 = _get_dummy_reference (self._num_true_conditions
-                                                          - self.__condition1 ().get ())
+                self.__condition2 = _get_dummy_reference (self._term_state & 2)
                 if self._has_signal () and isinstance (self.__condition1, DummyReference):
                     AbstractGCProtector.default.unprotect (self)
 
@@ -677,17 +822,18 @@ class _And (_Binary):
 
 
     def get (self):
-        return self._num_true_conditions == 2
+        return self._term_state == 3
 
-    def _on_term_change (self, new_state):
-        if new_state:
-            self._num_true_conditions += 1
-            if self._num_true_conditions == 2:
-                self._value_changed (True)
-        else:
-            self._num_true_conditions -= 1
-            if self._num_true_conditions == 1:
-                self._value_changed (False)
+
+    def _on_term1_change (self, new_state):
+        self._term_state ^= 1
+        if self._term_state & 2:
+            self._value_changed (new_state)
+
+    def _on_term2_change (self, new_state):
+        self._term_state ^= 2
+        if self._term_state & 1:
+            self._value_changed (new_state)
 
 
     def _get_operator_name (self):
@@ -701,17 +847,18 @@ class _Or (_Binary):
 
 
     def get (self):
-        return self._num_true_conditions > 0
+        return self._term_state != 0
 
-    def _on_term_change (self, new_state):
-        if new_state:
-            self._num_true_conditions += 1
-            if self._num_true_conditions == 1:
-                self._value_changed (True)
-        else:
-            self._num_true_conditions -= 1
-            if self._num_true_conditions == 0:
-                self._value_changed (False)
+
+    def _on_term1_change (self, new_state):
+        self._term_state ^= 1
+        if not self._term_state & 2:
+            self._value_changed (new_state)
+
+    def _on_term2_change (self, new_state):
+        self._term_state ^= 2
+        if not self._term_state & 1:
+            self._value_changed (new_state)
 
 
     def _get_operator_name (self):
@@ -725,15 +872,16 @@ class _Xor (_Binary):
 
 
     def get (self):
-        return self._num_true_conditions == 1
+        return self._term_state == 1 or self._term_state == 2
 
-    def _on_term_change (self, new_state):
-        if new_state:
-            self._num_true_conditions += 1
-        else:
-            self._num_true_conditions -= 1
 
-        self._value_changed (self._num_true_conditions == 1)
+    def _on_term1_change (self, new_state):
+        self._term_state ^= 1
+        self._value_changed (self._term_state == 1 or self._term_state == 2)
+
+    def _on_term2_change (self, new_state):
+        self._term_state ^= 2
+        self._value_changed (self._term_state == 1 or self._term_state == 2)
 
 
     def _get_operator_name (self):
@@ -741,6 +889,10 @@ class _Xor (_Binary):
 
 
 
+# Implementation note: `self.__term_state' is computed by a peculiar formula and many
+# functions depend on this way.  If you change the formula or `self.__term_state'
+# otherwise, you need to make adjustments in many places.
+
 class _IfElse (AbstractCondition):
 
     __slots__ = ('_IfElse__if', '_IfElse__then', '_IfElse__else', '_IfElse__term_state')
@@ -757,24 +909,34 @@ class _IfElse (AbstractCondition):
         self.__else       = weakref.ref (_else, self.__on_usage_change)
         self.__term_state = (_if.get () * 4 + _then.get () * 2 + _else.get ())
 
-        _if  .changed.connect (self.__on_term_change)
-        _then.changed.connect (self.__on_term_change)
-        _else.changed.connect (self.__on_term_change)
+        _if  .changed.connect (self.__on_if_term_change)
+        _then.changed.connect (self.__on_then_term_change)
+        _else.changed.connect (self.__on_else_term_change)
 
 
     def get (self):
         return _IfElse.__TERM_STATE_TO_SELF_STATE[self.__term_state]
 
 
-    def __on_term_change (self, new_state):
-        # FIXME: Is it efficient enough?
-        old_state         = _IfElse.__TERM_STATE_TO_SELF_STATE[self.__term_state]
-        self.__term_state = (  self.__if   ().get () * 4
-                             + self.__then ().get () * 2
-                             + self.__else ().get ())
-        new_state         = _IfElse.__TERM_STATE_TO_SELF_STATE[self.__term_state]
+    def __on_if_term_change (self, new_state):
+        self.__term_state ^= 4
+        state              = _IfElse.__TERM_STATE_TO_SELF_STATE[self.__term_state]
 
-        if  new_state != old_state:
+        if state != _IfElse.__TERM_STATE_TO_SELF_STATE[self.__term_state ^ 4]:
+            self._value_changed (state)
+
+
+    def __on_then_term_change (self, new_state):
+        self.__term_state ^= 2
+
+        if self.__term_state >= 4:
+            self._value_changed (new_state)
+
+
+    def __on_else_term_change (self, new_state):
+        self.__term_state ^= 1
+
+        if self.__term_state < 4:
             self._value_changed (new_state)
 
 
@@ -817,11 +979,17 @@ class _IfElse (AbstractCondition):
                     AbstractGCProtector.default.unprotect (self)
 
 
+    def __invert__(self):
+        # We don't create an object directly to include whatever optimizations might be
+        # there in if_else() method of `self.__if()'.
+        return self.__if ().if_else (self.__else (), self.__then ())
+
+
     def __repr__(self):
-        return '<%r if %r else %r>' % (self.__then, self.__if, self.__else)
+        return '<%r if %r else %r>' % (self.__then (), self.__if (), self.__else ())
 
     def __str__(self):
-        return '<%s if %s else %s>' % (self.__then, self.__if, self.__else)
+        return '<%s if %s else %s>' % (self.__then (), self.__if (), self.__else ())
 
 
 
