@@ -108,9 +108,9 @@ __all__       = ('AbstractSignal', 'Signal', 'CleanSignal')
 import sys
 import weakref
 
-from notify.bind  import *
-from notify.gc    import *
-from notify.utils import *
+from notify.bind  import Binding, WeakBinding
+from notify.gc    import AbstractGCProtector
+from notify.utils import raise_not_implemented_exception
 
 
 
@@ -166,7 +166,7 @@ class AbstractSignal (object):
             first invocation of C{L{accumulate_value}} method.  Default implementation
             returns C{None}.
 
-            @rtype: object
+            @rtype: C{object}
             """
 
             return None
@@ -176,7 +176,14 @@ class AbstractSignal (object):
             Accumulate C{value_to_add} into C{accumulated_value} and return the result.
             Result will be passed to next invocation of this method, if any.
 
-            @rtype: object
+            @param accumulated_value: value, returned from the last call to
+                                      C{accumulated_value} or C{L{get_initial_value}}.
+            @param value_to_add:      next handler value, to be added to
+                                      C{accumulated_value}.
+
+            @rtype:                   C{object}
+            @returns:                 New accumulated value, combination of
+                                      C{accumulated_value} and C{value_to_add}.
             """
 
             raise_not_implemented_exception (self)
@@ -186,8 +193,11 @@ class AbstractSignal (object):
             Examine C{accumulated_value} and decide if signal emission should continue.
             Default implementation always returns C{True}.
 
-            @rtype:   bool
-            @returns: Whether signal emission should continue.
+            @param accumulated_value: value, returned from the last call to
+                                      C{accumulated_value}.
+
+            @rtype:                   C{bool}
+            @returns:                 Whether signal emission should continue.
             """
 
             return True
@@ -200,7 +210,12 @@ class AbstractSignal (object):
             C{L{AbstractSignal.stop_emission}} method.  Default implementation does
             nothing and returns C{accumulated_value} unchanged.
 
-            @rtype: object
+            @param accumulated_value: value, returned from the last call to
+                                      C{accumulated_value} or C{L{get_initial_value}}.
+
+            @rtype:                   C{object}
+            @returns:                 Final accumulated value, either C{accumulated_value}
+                                      or some transformation of it.
             """
 
             return accumulated_value
@@ -309,7 +324,7 @@ class AbstractSignal (object):
         completely: if no one is listening, why emit at all?  This can be handy if
         computing emission arguments is not cheap.
 
-        @rtype:   bool
+        @rtype:   C{bool}
         @returns: C{True} if there are handlers or if there I{might} be handlers.
         """
 
@@ -319,7 +334,7 @@ class AbstractSignal (object):
         """
         Same as C{L{has_handlers}} method.
 
-        @rtype:   bool
+        @rtype:   C{bool}
         @returns: C{True} if there are handlers or if there I{might} be handlers.
         """
 
@@ -334,7 +349,7 @@ class AbstractSignal (object):
         when creating string representation of a signal or L{value
         <base.AbstractValueObject>} based on it.
 
-        @rtype:   int
+        @rtype:   C{int}
         @returns: Total number of connected handlers.
         """
 
@@ -347,7 +362,7 @@ class AbstractSignal (object):
         this method doesn’t detect if there are several handlers equal to C{handler}
         connected.
 
-        @rtype: bool
+        @rtype: C{bool}
         """
 
         raise_not_implemented_exception (self)
@@ -358,7 +373,7 @@ class AbstractSignal (object):
         Note that if there are several handlers equal to C{handler} connected, all are
         either blocked or non-blocked.
 
-        @rtype: bool
+        @rtype: C{bool}
         """
 
         raise_not_implemented_exception (self)
@@ -392,7 +407,7 @@ class AbstractSignal (object):
         connected yet), or does nothing.  See documentation of C{L{connect}} method for
         details.
 
-        @rtype:   bool
+        @rtype:   C{bool}
         @returns: C{True} if it has connected C{handler} with C{arguments}, C{False} if it
                   had been connected already.
         """
@@ -416,7 +431,7 @@ class AbstractSignal (object):
 
         This method I{must not} be called from outside.
 
-        @rtype: object
+        @rtype: C{object}
         """
 
         return WeakBinding.wrap (handler, arguments)
@@ -466,7 +481,7 @@ class AbstractSignal (object):
         need to call this method if the object is ‘thrown away’ already and handlers are
         ‘harmless’.  For details, please see C{L{WeakBinding}} class documentation.
 
-        @rtype:   bool
+        @rtype:   C{bool}
         @returns: C{True} if C{handler} has been disconnected; C{False} if it was not even
                   connected or had more than one connection.
         """
@@ -486,7 +501,7 @@ class AbstractSignal (object):
         need to call this method if the object is ‘thrown away’ already and handlers are
         ‘harmless’.  For details, please see C{L{WeakBinding}} class documentation.
 
-        @rtype:   bool
+        @rtype:   C{bool}
         @returns: C{True} if C{handler} has been disconnected; C{False} if it was not even
                   connected to begin with.
         """
@@ -513,7 +528,7 @@ class AbstractSignal (object):
         the same number of times as this method for a handler to be considered
         non-blocked.  This is usually what you want.
 
-        @rtype:   bool
+        @rtype:   C{bool}
         @returns: C{True} if C{handler} has been blocked, C{False} if it is not connected
                   to begin with.
         """
@@ -528,7 +543,7 @@ class AbstractSignal (object):
         must be unblocked exactly the same number of times as blocked, to become
         non-blocked again.
 
-        @rtype:   bool
+        @rtype:   C{bool}
         @returns: C{True} if C{handler} becomes non-blocked; C{False} if it is not even
                   connected or still remains blocked.
         """
@@ -547,7 +562,7 @@ class AbstractSignal (object):
         Note that if a given handler was connected to this signal with any arguments,
         C{arguments} to this method are I{appended} to those specified at connection time.
 
-        @rtype:   object
+        @rtype:   C{object}
         @returns: Value, determined by subclass and, possibly, by its
                   L{accumulator <AbstractAccumulator>}.
         """
@@ -558,7 +573,7 @@ class AbstractSignal (object):
         """
         Same as C{L{emit}} method.
 
-        @rtype:   object
+        @rtype:   C{object}
         @returns: Value, determined by subclass and, possibly, by its
                   L{accumulator <AbstractAccumulator>}.
         """
@@ -570,7 +585,7 @@ class AbstractSignal (object):
         Internal getter for the C{L{emission_level}} property.  Outside code should use
         that property, not the method directly.
 
-        @rtype: int
+        @rtype: C{int}
         """
 
         raise_not_implemented_exception (self)
@@ -580,7 +595,7 @@ class AbstractSignal (object):
         Internal getter for the C{L{emission_stopped}} property.  Outside code should use
         that property, not the method directly.
 
-        @rtype: bool
+        @rtype: C{bool}
         """
 
         raise_not_implemented_exception (self)
@@ -599,7 +614,7 @@ class AbstractSignal (object):
         ‘correct’ them, stop emission and immediately reemit signal with new, fixed
         arguments.
 
-        @rtype:   bool
+        @rtype:   C{bool}
         @returns: C{True} if this method stopped anything.
         """
 
@@ -698,6 +713,14 @@ class AbstractSignal (object):
         This handler is just a simple C{pass}.  It ignores everything, including
         C{SystemExit} and C{KeyboardInterrupt}.
 
+        @param signal:    signal, in which emission an exception was raised.
+        @type  signal:    C{AbstractSignal}
+
+        @param exception: raised exception object (same as C{sys.exc_info () [1]}.)
+
+        @param handler:   signal handler that rose the exception.
+        @type  handler:   callable
+
         @see:  exception_handler
         """
 
@@ -710,6 +733,14 @@ class AbstractSignal (object):
         C{sys.excepthook}.  Otherwise, exceptions are ignored and never reraised (except
         if reraised by C{sys.excepthook} itself.)
 
+        @param signal:    signal, in which emission an exception was raised.
+        @type  signal:    C{AbstractSignal}
+
+        @param exception: raised exception object (same as C{sys.exc_info () [1]}.)
+
+        @param handler:   signal handler that rose the exception.
+        @type  handler:   callable
+
         @see:  exception_handler
         """
 
@@ -721,6 +752,14 @@ class AbstractSignal (object):
         Handler for exceptions occured in signal handlers that reraises all exceptions.
         Regardless of exception type it is always thrown out of the emission call.
 
+        @param signal:    signal, in which emission an exception was raised.
+        @type  signal:    C{AbstractSignal}
+
+        @param exception: raised exception object (same as C{sys.exc_info () [1]}.)
+
+        @param handler:   signal handler that rose the exception.
+        @type  handler:   callable
+
         @see:  exception_handler
         """
 
@@ -794,6 +833,18 @@ class Signal (AbstractSignal):
 
 
     def __init__(self, accumulator = None):
+        """
+        Create a new C{Signal} with specified C{accumulator}.  By default, the signal will
+        not have any accumulator, so its C{L{emit}} method will always discard handlers’s
+        return values and return C{None}.
+
+        @param  accumulator: optional accumulator for signal handlers’ return values.
+        @type   accumulator: C{L{AbstractAccumulator}} or C{None}
+
+        @raises TypeError:   if C{accumulator} is not C{None} and not an instance of
+                             C{AbstractAccumulator}.
+        """
+
         if not (accumulator is None or isinstance (accumulator, Signal.AbstractAccumulator)):
             raise TypeError ("you must provide a `Signal.AbstractAccumulator' or None")
 
@@ -971,7 +1022,7 @@ class Signal (AbstractSignal):
 
             return True
 
-        except:
+        except ValueError:
             # It is not blocked to begin with.
             return False
 
