@@ -62,20 +62,28 @@ class BaseConditionTestCase (NotifyTestCase):
 class LogicConditionTestCase (NotifyTestCase):
 
     def test_not (self):
+        self.results = []
+
         condition = Condition (False)
 
         not_condition = ~condition
+        not_condition.store (self.simple_handler)
         self.assertEqual (not_condition.state, True)
 
         condition.state = True
         self.assertEqual (not_condition.state, False)
 
+        self.assert_results (True, False)
+
 
     def test_and (self):
+        self.results = []
+
         condition1 = Condition (False)
         condition2 = Condition (False)
 
         and_condition = condition1 & condition2
+        and_condition.store (self.simple_handler)
         self.assertEqual (and_condition.state, False)
 
         condition1.state = True
@@ -90,12 +98,17 @@ class LogicConditionTestCase (NotifyTestCase):
         condition2.state = True
         self.assertEqual (and_condition.state, True)
 
+        self.assert_results (False, True)
+
 
     def test_or (self):
+        self.results = []
+
         condition1 = Condition (False)
         condition2 = Condition (False)
 
         or_condition = condition1 | condition2
+        or_condition.store (self.simple_handler)
         self.assertEqual (or_condition.state, False)
 
         condition1.state = True
@@ -110,12 +123,17 @@ class LogicConditionTestCase (NotifyTestCase):
         condition2.state = True
         self.assertEqual (or_condition.state, True)
 
+        self.assert_results (False, True, False, True)
+
 
     def test_xor (self):
+        self.results = []
+
         condition1 = Condition (False)
         condition2 = Condition (False)
 
         xor_condition = condition1 ^ condition2
+        xor_condition.store (self.simple_handler)
         self.assertEqual (xor_condition.state, False)
 
         condition1.state = True
@@ -130,13 +148,18 @@ class LogicConditionTestCase (NotifyTestCase):
         condition2.state = True
         self.assertEqual (xor_condition.state, False)
 
+        self.assert_results (False, True, False, True, False)
+
 
     def test_if_else (self):
+        self.results = []
+
         condition1 = Condition (False)
         condition2 = Condition (False)
         condition3 = Condition (False)
 
         if_else_condition = condition1.if_else (condition2, condition3)
+        if_else_condition.store (self.simple_handler)
         self.assertEqual (if_else_condition.state, False)
 
         condition1.state = False
@@ -174,6 +197,57 @@ class LogicConditionTestCase (NotifyTestCase):
         condition3.state = True
         self.assertEqual (if_else_condition.state, True)
 
+        self.assert_results (False, True, False, True, False, True)
+
+
+
+class PredicateConditionTestCase (NotifyTestCase):
+
+    def test_predicate_condition_1 (self):
+        self.results = []
+
+        predicate = PredicateCondition (bool, None)
+        predicate.store (self.simple_handler)
+
+        predicate.update (10)
+
+        self.assert_results (False, True)
+
+
+    def test_predicate_condition_2 (self):
+        self.results = []
+
+        predicate = PredicateCondition (bool, None)
+        predicate.store (self.simple_handler)
+
+        predicate.update (False)
+
+        self.assert_results (False)
+
+
+    def test_predicate_condition_3 (self):
+        self.results = []
+
+        predicate = PredicateCondition (lambda x: x > 10, 0)
+        predicate.store (self.simple_handler)
+
+        predicate.update (10)
+        predicate.update (20)
+        predicate.update (-5)
+
+        self.assert_results (False, True, False)
+
+
+    def test_predicate_condition_error_1 (self):
+        self.assertRaises (TypeError, lambda: PredicateCondition (None, None))
+
+
+    def test_predicate_condition_error_2 (self):
+        self.assertRaises (ZeroDivisionError, lambda: PredicateCondition (lambda x: 1/x, 0))
+
+        predicate = PredicateCondition (lambda x: 1/x, 10)
+        self.assertRaises (ZeroDivisionError, lambda: predicate.update (0))
+
 
 
 class WatcherConditionTestCase (NotifyTestCase):
@@ -187,6 +261,8 @@ class WatcherConditionTestCase (NotifyTestCase):
         condition = Condition (True)
         watcher.watch (condition)
 
+        self.assert_(watcher.watched_condition is condition)
+
         condition.state = False
 
         self.assert_results (False, True, False)
@@ -206,7 +282,9 @@ class WatcherConditionTestCase (NotifyTestCase):
         watcher.watch (condition3)
         watcher.watch (None)
 
-        # Later two watch() calls must not change watcher's state.
+        self.assert_(watcher.watched_condition is None)
+
+        # Last two watch() calls must not change watcher's state.
         self.assert_results (True, False)
 
 
@@ -223,7 +301,8 @@ class WatcherConditionTestCase (NotifyTestCase):
         condition = Condition (True)
         watcher   = WatcherCondition (condition)
 
-        self.assertRaises (TypeError, lambda: watcher.watch (watcher))
+        self.assertRaises (ValueError, lambda: watcher.watch (watcher))
+        self.assert_      (watcher.watched_condition is condition)
 
 
 
