@@ -80,25 +80,6 @@ class BaseVariableTestCase (NotifyTestCase):
         self.assert_(is_true)
 
 
-    def test_is_not_empty (self):
-        variable     = Variable ()
-        is_not_empty = variable.is_not_empty ()
-
-        self.assert_(not is_not_empty)
-
-        variable.value = (1, 2, 3)
-        self.assert_(is_not_empty)
-
-        variable.value = 'a string'
-        self.assert_(is_not_empty)
-
-        variable.value = []
-        self.assert_(not is_not_empty)
-
-        variable.value = ''
-        self.assert_(not is_not_empty)
-
-
     def test_is_allowed_value (self):
 
         class PositiveVariable (Variable):
@@ -132,6 +113,8 @@ class WatcherVariableTestCase (NotifyTestCase):
         variable = Variable ('abc')
         watcher.watch (variable)
 
+        self.assert_(watcher.watched_variable is variable)
+
         variable.value = 60
 
         self.assert_results (None, 'abc', 60)
@@ -151,6 +134,8 @@ class WatcherVariableTestCase (NotifyTestCase):
         watcher.watch (variable3)
         watcher.watch (None)
 
+        self.assert_(watcher.watched_variable is None)
+
         # Later two watch() calls must not change watcher's value.
         self.assert_results ([], 'string', None)
 
@@ -168,7 +153,8 @@ class WatcherVariableTestCase (NotifyTestCase):
         variable = Variable ()
         watcher  = WatcherVariable (variable)
 
-        self.assertRaises (TypeError, lambda: watcher.watch (watcher))
+        self.assertRaises (ValueError, lambda: watcher.watch (watcher))
+        self.assert_      (watcher.watched_variable is variable)
 
 
 
@@ -233,6 +219,19 @@ class VariableDerivationTestCase (NotifyTestCase):
         self.assertRaises (ValueError, lambda: variable.set ([]))
 
 
+    def test_derivation_5 (self):
+        IntVariable = Variable.derive_type ('IntVariable',
+                                            allowed_value_types = int, default_value = 10)
+
+        variable = IntVariable ()
+        self.assertEqual (variable.value, 10)
+
+        variable = IntVariable (30)
+        self.assertEqual (variable.value, 30)
+
+        self.assertRaises (ValueError, lambda: variable.set ('string'))
+
+
     def test_multiple_derivation (self):
         # Derive two types and make sure they don't spoil each other's is_allowed_value()
         # method.
